#!/usr/bin/python
import getopt
import sys

split_size=10000
overlap_size=100

def error(x):
	sys.stderr.write(x)


def usage():
	print '''Usage: %s ...

This script merges VARiD results (pretty base format files) over sub contigs created by splitgenome.py into one 
VARiD output (pretty base format) file.

Parameters:

--input-prefix prefix
	The prefix used for the sub contigs generated by splitgenome.py. If the prefix is X, files X.0, X.1,
	... , X.(contig-count - 1) will be read.	

--output filename
	The output file of the merged VARiD (pretty base format) results. Default value is "-", which is stdout.
	
--contig-count number
	The number of contigs that the original file was split into.

--split-size number
	The split-size used when called splitgenome.py. Default value is %d.

--overlap-size number
	The overlap-size used when called splitgenome.py. Default value is %d.

''' % (sys.argv[0], split_size, overlap_size)
	sys.exit(1)

if __name__=='__main__':
	try:
			opts, args = getopt.getopt(sys.argv[1:], "",\
			   ["input-prefix=","output=","contig-count=","split-size=","overlap-size="])
	except getopt.GetoptError, err:
		print >> sys.stderr, str(err)
		usage()
		sys.exit(1)

	input_prefix=""
	output_filename="-"
	contig_count=-1
	for o,a in opts:
		if o in ("--input-prefix"):
			input_prefix=a
		elif o in ("--output"):
			output_filename=a
		elif o in ("--contig-count"):
			contig_count=int(a)
		elif o in ("--split-size"):
			split_size=int(a)
		elif o in ("--overlap-size"):
			overlap_size=int(a)
		else:
			print "%s is an invalid option!" % o
			usage()
			sys.exit(1)

	if input_prefix=="":
		print "Please specify a input-prefix"
		usage()
		sys.exit(1)
	if contig_count==-1:
		print "Invalid contig-count, %d" % contig_count
		usage()
		sys.exit(1)

	if output_filename == '-':
		output_file = sys.stdout
	else:
		output_file = open(output_filename,'w')
	for x in range(contig_count):
		try:
			infile = open(input_prefix + '.' + str(x),'r')
			for line in infile:
				s = line.split('\t')
				w = s[0].split('_')
				contignum = int(w[-2])
				base = int(w[-1])
				if contignum == 0 and base < split_size:
					s[0] = '_'.join(w[:-2] + [str(base)])
					output_file.write('\t'.join(s))
				if contignum != 0 and 0 <= base - (overlap_size / 2) < split_size:
					base = base - (overlap_size / 2) + (contignum * split_size)
					s[0] = '_'.join(w[:-2] + [str(base)])
					output_file.write('\t'.join(s))
				if contignum == (contig_count-1) and base - (overlap_size / 2) == split_size:
					base = base - (overlap_size / 2) + (contignum * split_size)
					s[0] = '_'.join(w[:-2] + [str(base)])
					output_file.write('\t'.join(s))
			infile.close()
		except IOError, err:
			print "Cannot open file, \"%s\"!" % (input_prefix + '.' + str(x)) 
			print err
			sys.exit(1)
			pass
		
