#!/usr/bin/python

import sys
import getopt

split_size=10000
overlap_size=100

def error(x):
	sys.stderr.write(x)

def usage():
	print '''Usage: %s ...

This script takes in a fasta file a split size and an overlap size. The resulting files are such that each
file contains no more then 'split size' base pairs of the original file. Also for every two consecutive 
files generated they share 'overlap size' base pairs in common at the begining and end of the two files. 

Parameters:

--input filename
	The input mfa file. Default is "-" which is stdin.

--output-prefix prefix
	Prefix to use for output files.

--split-size number
	The maximum number of base pairs in each sub contig. Default value is %d.

--overlap-size number
	The number of bp to overlap between sub contigs. Default value is %d.

Output:
	The number of 'sub contig' files created.

''' % (sys.argv[0],split_size,overlap_size)

if __name__=='__main__':
	try:
		opts, args = getopt.getopt(sys.argv[1:], "",\
		   ["input=","output-prefix=","split-size=","overlap-size="])
	except getopt.GetoptError, err:
		print >> sys.stderr, str(err)
		usage()
		sys.exit(1)

	input_filename="-"
	output_prefix=""
	for o,a in opts:
		if o in ("--input"):
			input_filename=a
		elif o in ("--output-prefix"):
			output_prefix=a
		elif o in ("--split-size"):
			split_size=int(a)
		elif o in ("--overlap-size"):
			overlap_size=int(a)
		else: 
			print "%s is an invalid option!" % o
			usage()
			sys.exit(1)

	if output_prefix=="":
		print "Please specify an output prefix"
		usage()
		sys.exit(1)	

	
	if input_filename == '-':
		input_file = sys.stdin
	else:
		input_file = open(input_filename,'r')
		
	
	#read the header line
	header = input_file.readline()[:-1]
	if header[0] == '>':
		name = header[1:]
	else:
		error('no header found')
		sys.exit(1)
	
	last = ''
	lastcount = 0
	contigcount = 0
	basecount = 0
	writecount = 0
	outfile = None

	for line in input_file:
		line = line[:-1]
		if line[0] == '>':
			name = line[1:]
			if writecount % 80 != 0:
				outfile.write('\n')
			last = ''
			lastcount = 0
			contigcount = 0
			basecount = 0
			writecount = 0
 			continue
		for c in line:
			if basecount == split_size + (overlap_size / 2) or basecount == 0:
				if basecount != 0:
					outfile.write('\n')
				if outfile:
					outfile.close()
				outfile = open(output_prefix + '.' + str(contigcount),'w')
				outfile.write('>' + name + '_' + str(contigcount) + '\n')
				contigcount += 1
				if last:
					outfile.write(last)
					basecount = lastcount/2
					writecount = lastcount
					last = ''
					lastcount = 0
			if basecount >= split_size - (overlap_size / 2):
				last += c
				lastcount += 1
				if lastcount % 80 == 0:
					last += '\n'
			if basecount < split_size + (overlap_size / 2):
				outfile.write(c)
				basecount += 1
				writecount += 1
				if writecount % 80 == 0:
					outfile.write('\n')
	
	if writecount % 80 != 0:
		outfile.write('\n')

	outfile.close()
	print contigcount
