#!/usr/bin/python

import getopt
import sys


split_size=10000
overlap_size=100

def error(x):
	sys.stderr.write(x)


def usage():
	print '''Usage: %s ...

This script takes a sorted list of alignments and splits them into smaller files based on the 
output files of splitgenome.py. The alignments are broken apart into regions of maximum 
'split size' base pairs with 'overlap size' base pairs in common between consecutive parts. 

--sam
	Use this flag is the alignments are in SAM format.

--alginments filename
	The alignments filename.

--output-prefix prefix
	The output prefix for alignment files.

--split-size number
	The split size. Default is %d.

--overlap-size number
	The overlap size. Default is %d.

	''' % (sys.argv[0],split_size,overlap_size)

if __name__=='__main__':

	try:
		opts, args = getopt.getopt(sys.argv[1:], "",\
		   ["sam","alignments=","output-prefix=","split-size=","overlap-size="])
	except getopt.GetoptError, err:
		print >> sys.stderr, str(err)
		usage()
		sys.exit(1)

	mod=0
	useSam=False
	output_prefix=""
	alignments_filename="-"
	for o,a in opts:
		if o in ("--alignments"):
			alignments_filename=a
		elif o in ("--output-prefix"):
			output_prefix=a
		elif o in ("--split-size"):
			split_size=int(a)
		elif o in ("--overlap-size"):
			overlap_size=int(a)
		elif o in ("--sam"):
			mod=1
			useSam=True
		else:
			print "%s is an invalid option" % o
			usage()
			sys.exit(1)

	if output_prefix=="":
		print "Please specify a output-prefix"
		usage()
		sys.exit(1)
	if alignments_filename=="":
		print "Please specify a alignments filename"
		usage()
		sys.exit(1)

	if alignments_filename=="-":
		alignments_file=sys.stdin
	else:
		alignments_file=open(alignments_filename,'r')
	
	
	last = ''
	laststart = 0
	lastend = 0
	prevstart = 0
	prevend = 0
	swap = False

	header = ''
	next = ''
	filenum = -1
	outfile = None
		
	for line in alignments_file:
		if useSam:
			if line[0] == '@' or line[0]=='\n':
				header += line
			else:
				s = line.split('\t')
				s[0] = s[0].replace(' ','_')
				contigname = s[2]
				contigstart = int(s[3])
				contigstartnum = contigstart / split_size
				contigstartpos = contigstart % split_size
				
				if contigstartpos > split_size - (overlap_size / 2):
					s[2] = contigname + '_' + str(contigstartnum + 1)
					start = contigstartpos - ( split_size - (overlap_size / 2))
					s[3] = str(start)
					next += '\t'.join(s)#+'\n'
				
				s[2] = contigname + '_' + str(contigstartnum)
				start = contigstartpos
				if contigstartnum != 0:
					start += overlap_size / 2
				s[3] = str(start)
				if contigstartnum != filenum:
					if outfile:
						outfile.close()
					if next:
						filenum += 1
						outfile = open(output_prefix + '.' + str(filenum),'w')
						outfile.write(header)
						outfile.write(next)
						next=''

					if contigstartnum != filenum:
						if outfile:
							outfile.close()
						filenum = contigstartnum
						outfile = open(output_prefix + '.' + str(filenum),'w')
						outfile.write(header)
				outfile.write('\t'.join(s))
						
				
		else:
			if line[0] == '>':
				if(last):
					next += last
					last = ''
				s=line.split('\t')
				s[0] = s[0].replace(' ','_')
				contigstart = int(s[3])
				contigend = int(s[4])
				swap = s[2] == '-'
				diff = contigend - contigstart
				
				if not diff < overlap_size / 2:
					print 'too small overlap size'
					print diff, overlap_size
					print line
					exit(1)
				
				contigname = s[1]
			
				contigstartnum = contigstart / split_size
				contigendnum = contigend / split_size
				contigstartpos = contigstart % split_size
				contigendpos = contigend % split_size
			
				if contigstartpos > split_size - (overlap_size / 2):
					s[1] = contigname + '_' + str(contigstartnum + 1)
					start = contigstartpos - ( split_size - (overlap_size / 2))
					s[3] = str(start)
					s[4] = str(start + diff)
					laststart = start
					lastend = start + diff
					last = '\t'.join(s)
			
				s[1] = contigname + '_' + str(contigstartnum)
				start = contigstartpos
				if contigstartnum != 0:
					start += overlap_size / 2
				s[3] = str(start)
				s[4] = str(start + diff)
				prevstart = start
				prevend = start + diff
			
				if contigstartnum != filenum:
					if outfile:
						outfile.close()
					if next:
						filenum += 1
						outfile = open(output_prefix + '.' + str(filenum),'w')
						outfile.write(header)
						outfile.write(next)
						next=''

					if contigstartnum != filenum:
						if outfile:
							outfile.close()
						filenum = contigstartnum
						outfile = open(output_prefix + '.' + str(filenum),'w')
						outfile.write(header)
				outfile.write('\t'.join(s))
			
			elif line[0] == 'G':
				digits = 0
				writestart = True
				for c in line:
					if ord('0') <= ord(c) <= ord('9'):
						digits += 1
					else:
						if digits:
							if writestart != swap:
								writestart = False
								if last:
									last += str(laststart).rjust(digits)
								outfile.write(str(prevstart).rjust(digits))
							else:
								if last:
									last += str(lastend).ljust(digits)
								outfile.write(str(prevend).ljust(digits))
								writestart = False
							
							digits = 0
						outfile.write(c)
						if last:
							last += c
			elif line[0] == '#':
				header += line
			else:
				outfile.write(line)
				if last:
					last += line
