#!/bin/sh
if [ -z "$VARID_FOLDER" ]; then
  echo 1>&2 "warning: VARID_FOLDER undefined; assuming ."
  VARID_FOLDER="."
fi

if [ ! -d "$VARID_FOLDER/helper/" ]; then
  echo 1>&2 "error: cannot find directory $VARID_FOLDER/helper/"
  exit 1
fi

SPLIT_SIZE=10000
OVERLAP_SIZE=100


function usage { 
	echo $0 [ --sam ] [ --alignments filename ] [ --reference filename ] [ --donor-name name ] [ --output filename ] [ --temp-dir path ] [ --split-size 10000 \{default\} ] [ --overlap-size \{default 100\} ]
}

if [ $# -lt 5 ]
then
	usage
	exit 1
fi

SAM="no"
until [ -z "$1" ]; do
  case $1 in
    "--alignments")
      shift
      ALIGNMENTS_FILE=$1; shift;;
    "--reference")
      shift
      REFERENCE_FILE=$1; shift;;
    "--sam")
      shift
      SAM="yes";;
    "--output")
      shift
      OUTPUT_FILE=$1; shift;;
    "--overlap-size")
      shift
      OVERLAP_SIZE=$1; shift;;
    "--split-size")
      shift
      SPLIT_SIZE=$1; shift;;
    "--donor-name")
      shift
      DONOR_NAME=$1; shift;;
    "--temp-dir")
      shift
      TEMP_DIR=$1; shift;;
    *) 
	usage
	echo Option $1 does not exist!
	exit 1
	break;;
  esac
done


if [ -z "${ALIGNMENTS_FILE}" -o -z "${REFERENCE_FILE}" -o -z "${OUTPUT_FILE}" -o -z "${TEMP_DIR}" ]; then 
	usage
	exit 1
fi

mkdir -p ${TEMP_DIR}

SAM_FLAG=""
if [ "$SAM" = "yes" ]
then
	SAM_FLAG="--sam"
	echo running "sort -t$'\t' -k 4n ${ALIGNMENTS_FILE} > ${TEMP_DIR}/`basename ${ALIGNMENTS_FILE}`.tmp"
	sort -t$'\t' -k 4n ${ALIGNMENTS_FILE} > ${TEMP_DIR}/`basename ${ALIGNMENTS_FILE}`.tmp
fi

echo running "python ${VARID_FOLDER}/helper/splitreads_helper.py ${SAM_FLAG} --alignments ${TEMP_DIR}/`basename ${ALIGNMENTS_FILE}`.tmp --output-prefix ${TEMP_DIR}/`basename ${ALIGNMENTS_FILE}`.tmp --split-size ${SPLIT_SIZE} --overlap-size ${OVERLAP_SIZE}"
python ${VARID_FOLDER}/helper/splitreads_helper.py ${SAM_FLAG} --alignments ${TEMP_DIR}/`basename ${ALIGNMENTS_FILE}`.tmp --output-prefix ${TEMP_DIR}/`basename ${ALIGNMENTS_FILE}`.tmp --split-size ${SPLIT_SIZE} --overlap-size ${OVERLAP_SIZE}

echo running "python ${VARID_FOLDER}/helper/splitgenome.py --input ${REFERENCE_FILE} --output-prefix ${TEMP_DIR}/`basename ${REFERENCE_FILE}`.tmp --split-size ${SPLIT_SIZE} --overlap-size ${OVERLAP_SIZE}"
contigcount=$(python ${VARID_FOLDER}/helper/splitgenome.py --input ${REFERENCE_FILE} --output-prefix ${TEMP_DIR}/`basename ${REFERENCE_FILE}`.tmp --split-size ${SPLIT_SIZE} --overlap-size ${OVERLAP_SIZE})

i=0
while [ $i -lt $contigcount ]
do
	echo running "${VARID_FOLDER}/varid ${SAM_FLAG} --alignments=${TEMP_DIR}/`basename ${ALIGNMENTS_FILE}`.tmp.$i --ref-file=${TEMP_DIR}/`basename ${REFERENCE_FILE}`.tmp.$i --donor-name=${DONOR_NAME} --temp-dir=${TEMP_DIR} --output=${TEMP_DIR}/`basename ${OUTPUT_FILE}`.tmp.$i"
	${VARID_FOLDER}/varid ${SAM_FLAG} --alignments=${TEMP_DIR}/`basename ${ALIGNMENTS_FILE}`.tmp.$i --ref-file=${TEMP_DIR}/`basename ${REFERENCE_FILE}`.tmp.$i --donor-name=${DONOR_NAME} --temp-dir=${TEMP_DIR} --output=${TEMP_DIR}/`basename ${OUTPUT_FILE}`.tmp.$i
	i=`expr $i + 1`
done

echo running "python ${VARID_FOLDER}/helper/mergeresults.py --input-prefix ${TEMP_DIR}/`basename ${OUTPUT_FILE}`.tmp --output ${OUTPUT_FILE} --contig-count $contigcount --split-size ${SPLIT_SIZE} --overlap-size ${OVERLAP_SIZE}"
python ${VARID_FOLDER}/helper/mergeresults.py --input-prefix ${TEMP_DIR}/`basename ${OUTPUT_FILE}`.tmp --output ${OUTPUT_FILE} --contig-count $contigcount --split-size ${SPLIT_SIZE} --overlap-size ${OVERLAP_SIZE}
