/*
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef VARID_H
#define VARID_H

#include "va_types.h"

void varid_single(
    const char   * readsfile,
    const char   * reffile,
    const char   * don_name,
    VA_VecDouble   thr_vec_p,
    const char   * path,
    int 		   useSam);

void split_reads(
    const char  * readsfile,
    char       ** filenames,
    SnpGroups_t * snpgroup,
    VA_UINT8      method,
    VA_INT32   ** fid_lines,
    VA_INT32   ** fid_reads,
    int			  useSam,
    VA_Vector     fastaVec);

void get_reads(
    char       * mapfile,
    VA_UINT32    nr_reads,
    VA_UINT32    nr_lines,
    VA_Vector  * outreads,
    VA_VecI32  * outinsvec);

void process_read(
    CurrReads_t * curr_read,
    ReadsNode_t * read,
    VA_INT32    * rstart,
    VA_INT32    * skipread,
    VA_VecI32   * insvec);

VA_Vector align_reads(
    VA_VecU8  ref,
    VA_Vector reads,
    VA_VecI32 ins_vec);

CData_t real_cdata_prep(
    int       doSim,
    int       isDip);

void varid(
    CData_t        cdata,
    VA_INT32       ref_pos,
    Maps_t       * maps,
    Covmaps_t    * covmap,
    VA_MatDouble * p,
    int			   useSam);

void get_transitions(
    Frequency_t    freq,
    Comb_t         comb,
    VA_MatDouble * big_mat,
    Maps_t       * maps);

Covmaps_t varid_covmaps(
    CData_t cdata,
    int useSam);

VA_MatDouble varid_emissions(
    CData_t      cdata,
    VA_MatDouble trans_mat,
    Covmaps_t    covmap,
    Comb_t       comb);

VA_DOUBLE varid_forward(
    VA_MatDouble   trans_mat,
    CData_t        cdata,
    VA_MatDouble   e,
    Maps_t         maps,
    VA_VecI32      window,
    VA_MatDouble * f,
    VA_VecU8     * skip_pos,
    VA_VecDouble * s);

VA_DOUBLE varid_backward(
    VA_MatDouble   trans_mat,
    CData_t        cdata,
    VA_MatDouble   e,
    Maps_t         maps,
    VA_VecI32      window,
    VA_MatDouble * b,
    VA_VecU8     * skip_pos,
    VA_VecDouble * s);

void predict(
    VA_MatDouble p,
    Maps_t         maps,
    VA_MatDouble * maxvals_2,
    VA_MatU8     * lmat,
    VA_MatU8     * cmat);

void destroy_state(
    CData_t        cdata,
    VA_MatU8       cmat,
    VA_Vector      reads,
    size_t         maxreadsize,
    VA_VecI32      readstarts,
    Covmaps_t      covmap,
    VA_VecDouble   thr_vec,
    VA_MatU8       lmat,
    VA_MatU8     * ch_list,
    VA_MatDouble * tot,
    VA_MatDouble * top2,
    int			   useSam);

void get_pred(
    Covmaps_t      covmap,
    Maps_t         maps,
    VA_MatU8       ch_list,
    VA_VecDouble   thr_vec_p,
    VA_MatDouble   maxvals_2,
    VA_MatU8       lmat,
    VA_VecU8       hap1,
    VA_VecU8       hap2,
    VA_VecU8     * pred1_whole,
    VA_VecU8     * pred2_whole);

VA_InternalStats compare_grp(
    VA_Vector    rha_real,
    VA_VecU8     hap1,
    VA_VecU8     hap2,
    VA_VecU8     ref,
    Covmaps_t    covmap);

void writePrettybase(
    VA_UINT32    startpos,
    const char * ref_name,
    const char * don_name,
    VA_VecU8     hap1,
    VA_VecU8     hap2,
    VA_VecU8     ref,
    const char * filename);

VA_VecU8 va_reallocVecU8(
    VA_VecU8 vec,
    size_t   n);

VA_VecU32 va_reallocVecU32(
    VA_VecU32 vec,
    size_t    n);

VA_VecI32 va_reallocVecI32(
    VA_VecI32 vec,
    size_t    n);

VA_VecU8 va_allocVecU8(
    size_t  size);

VA_VecI32 va_allocVecI32(
    size_t  size);

VA_VecU32 va_allocVecU32(
    size_t  size);

VA_VecDouble va_allocVecDouble(
    size_t  size);

VA_MatDouble va_allocMatDouble(
    size_t  size_x,
    size_t  size_y);

VA_MatI32 va_allocMatI32(
    size_t  size_x,
    size_t  size_y);

VA_MatU32 va_allocMatU32(
    size_t  size_x,
    size_t  size_y);

VA_MatU8 va_allocMatU8(
    size_t  size_x,
    size_t  size_y);

void fliplrU8(
    VA_VecU8 toFlip);

void fliplrDouble(
    VA_VecDouble toFlip);

int va_readfasta(
    const char * reffile,
    VA_Vector  * vfasta);

VA_VecI32 va_sortVecI32(
    VA_VecI32 vec);

void va_loadMatDouble(
    char         * filename,
    VA_MatDouble * p);

void va_saveMatDouble(
    char       * filename,
    VA_MatDouble p);

VA_VecU8 nuc2col(
    VA_VecU8 innuc);

char * va_basename(
    const char * file);

VA_VecU8 col2nuc(
    VA_UINT8 startletter,
    VA_VecU8 colorref);

VA_VecU8 nuc2nuc(
    VA_VecU8 innuc);

VA_VecU8 get_complement(
    VA_VecU8 str1);

void va_load(
    const char * filename,
    VA_Vector  *  reads,
    VA_VecI32  * insvec);

void va_save(
    const char  * filename,
    VA_Vector     reads,
    VA_VecI32     insvec);

VA_UINT8 * nuc2nr(
    const char * str,
    size_t       sz);

VA_VecI32 va_unique(
    VA_VecI32 ref);

#endif

