/*
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "va_log.h"
#include "va_assert.h"
#include "varid.h"




extern double*  cs_error_cnt;
extern double*  cs_error_dpts;
extern unsigned int cs_error_size;
extern double*  ls_error_cnt;
extern double*  ls_error_dpts;
extern unsigned int ls_error_size;

char* lets="ACGT";
char* nums="0123";

long int strchri (char* str, int c) {
	return (long int) strchr(str, toupper(c));
}

int follows(char prev, char color, char next) {
	int a,b,c;
	if (!strchri(lets, prev) || !strchri(lets, next) || !strchri(nums, color))
		return 0;
	a =  strchri(lets, prev) - (long int) lets;
	b =  strchri(lets, next) - (long int) lets;
        c =  strchri(nums, color) - (long int) nums;
	if ((a ^ c) == b)
		return 1;
	return 2;
}

void learn_err_mat(char* ls_string, char* cs_string, char* gen_string, int read_start, int read_end, int read_length) {
	int i,x;
	if (cs_string) {
                if (read_length > cs_error_size) {
                        cs_error_cnt = (double*) realloc(cs_error_cnt, read_length * sizeof(double));
                        cs_error_dpts = (double*) realloc(cs_error_dpts, read_length * sizeof(double));
			int j;
			for (j=cs_error_size; j<read_length; j++) {
				cs_error_cnt[j]=cs_error_dpts[j]=0;
			}
                        cs_error_size = read_length;
                }
		i = read_start-1;
		if (read_start == 1) {
			x = follows(cs_string[0], cs_string[i+1], ls_string[i]);
			if (x > 1)  cs_error_cnt[i]++;				
			if (x > 0)  cs_error_dpts[i]++;
		}	
		for (i = read_start; i < read_end-1; i++) {
			x = follows(ls_string[i-1], cs_string[i+1], ls_string[i]);
			if (x > 1) cs_error_cnt[i]++;
			if (x > 0) cs_error_dpts[i]++;
		}

	}
	else {
		if (read_length > ls_error_size) {
			ls_error_cnt = (double*) realloc(ls_error_cnt, read_length * sizeof(double));
                        ls_error_dpts = (double*) realloc(ls_error_dpts, read_length * sizeof(double));
			int j;
			for (j=ls_error_size; j<read_length; j++) {
				ls_error_cnt[j]=ls_error_dpts[j]=0;
			}
			ls_error_size = read_length;
		}
		for (i = read_start-1; i < read_end -1; i++) {
			if (ls_string[i] != gen_string[i]) ls_error_cnt[i]++;
			ls_error_dpts[i]++;
		}	
	
	}
	
	
}





static void getIns(
   VA_VecU8    chr,
   VA_UINT32   chrstart,
   VA_UINT8    corr,
   VA_VecI32 * ins,
   VA_VecI32 * ins_local);

void process_read(
    CurrReads_t * curr_read, 
    ReadsNode_t * read,
    VA_INT32    * rstart,
    VA_INT32    * skipread,
    VA_VecI32   * insvec)
{
    char readname[BUFSIZ];
    char contigname[BUFSIZ];
    char tline[BUFSIZ],readstring[BUFSIZ],genomestring[BUFSIZ];
    char strand;
    int chrstart;
    int chrend;
    int readstart;
    int readend;
    int readlength;
    int score;
    char editstring[BUFSIZ];
    double normodds;
    double pgenome;
    double pchance;
    //char * scan_format = ">%s%s %c %d %d %d %d %d %d %s %lf %lf %lf";
    char * scan_Rline = "R: %d %s %d";
    char * scan_Tline = "T: %s";
    char * scan_Gline = "G: %d %s %d";
    char * scan_format = ">%s\t%s\t%c\t%d\t%d\t%d\t%d\t%d\t%d\t%s\t%lf\t%lf\t%lf";
    //char * scan_Rline = "R:%d%*[ ]%s%d";
    //char * scan_Gline = "G:%d%s%d";
    int startend_1;
    int startend_2;
    char rd_t[BUFSIZ];
    char chr[BUFSIZ];
    VA_VecU8 read_t;
    VA_VecU8 trans;
    VA_VecU8 seq;
    VA_VecI32 ins_assert;
    VA_VecI32 ins;
    VA_VecU8 chr_assert;
    int nr_gaps = 0;
    int i;
    int n;
    int isLetterspace;
    int isForward;

    skipread = FALSE;

    char* save_pointer;
    char* str=curr_read->sline;
    char* tok;
    int parsed_tokens=0;
    while ( (tok=strtok_r(str,"\t",&save_pointer))!=NULL && parsed_tokens < 13) {
	switch(parsed_tokens) {
		case 0:	strcpy(readname,tok); break;
		case 1: strcpy(contigname,tok); break;
		case 2: strand=tok[0]; break;		
		case 3: chrstart=atoi(tok); break;
		case 4: chrend=atoi(tok); break;
		case 5: readstart=atoi(tok); break;
		case 6: readend=atoi(tok); break;
		case 7: readlength=atoi(tok); break;
		case 8: score=atoi(tok); break;
		case 9: strcpy(editstring,tok); break;
		case 10: normodds=atof(tok); break;
		case 11: pgenome=atof(tok); break;
		case 12: pchance=atof(tok); break;
		default: exit(1);
	}
	str=NULL;
	parsed_tokens++;

    }
    VA_ASSERT(strand=='-' || strand=='+');
    VA_ASSERT(parsed_tokens==13);
    //int res= sscanf(curr_read->sline, scan_format, readname, contigname, &strand,
      //  &chrstart, &chrend, &readstart, &readend, &readlength, 
        //&score, editstring, &normodds, &pgenome, &pchance);
    /* start the work */
	int res;
    /* get the rest of the values we care about */
    read->name = (char *)malloc(strlen(readname) + 1);
    VA_ASSERT(read->name != NULL);
    strcpy(read->name, readname);

    read->strand = strand;
    read->chrstart = chrstart;
    read->chrend = chrend;
    read->extras.score = score;
    read->extras.nodds = normodds;
    read->extras.pgenome = pgenome;
    read->extras.pchance = pchance;

    /* for sorting and flagging found reads */
    *rstart = chrstart;
    read_t.data = rd_t;
 
    /* Compute the read */
    res = sscanf(curr_read->Rline, scan_Rline, &startend_1, read_t.data, &startend_2);
    strcpy(readstring,read_t.data);
    read_t.size = strlen(read_t.data);
	size_t readsize = read_t.size - 1;
	char *c = read_t.data;
	while (*c != '\0'){
		if (*c == '-') readsize--;
		c++;
	}
	//fprintf(stderr,"\n\nres=%i, Rline: %s\n",res,curr_read->Rline);
	//fprintf(stderr,"Gline: %s\n",curr_read->Gline);
	//fprintf(stderr,"startend_1=%i, readstart=%i\n",startend_1,readstart);
    VA_ASSERT(startend_1 == readstart);
    //fprintf(stderr,"startend_2=%i, readend=%i\n",startend_2,readend);
    VA_ASSERT(startend_2 == readend);

    /* detect the type of read (assuming no '-' at the beginning of a read) */
    
    for (i = 1; i < read_t.size; i++)
        if (read_t.data[i] != '-')
            break;

    VA_ASSERT(i < read_t.size);
           
    chr[0] = read_t.data[i];
    isLetterspace = FALSE;
    read->seqtype = COLORSPACE_READ;

    if ((chr[0] == 'A') || (chr[0] == 'C') || 
        (chr[0] == 'T') || (chr[0] == 'G') || (chr[0] == 'N'))
    {
        isLetterspace = TRUE;
        read->seqtype = LETTERSPACE_READ;
    }

    /* deal with the read */
    if (isLetterspace)
    {
        VA_VecU8 rd_res_t;
        char * p = read_t.data;
        size_t s1;
        VA_VecI32 fr = va_allocVecI32(read_t.size);
      
        fr.size = 0;

        /* verify method */
        for (i = 1; i <= read_t.size; i++)
            if (read_t.data[i - 1] != '-')
                fr.data[fr.size++] = i;
	//fprintf(stderr,"read.data=%s\n",read_t.data);
	//fprintf(stderr,"startend_2=%i, fr.size=%i, startend_1=%i\n",startend_2,fr.size,startend_1);
        VA_ASSERT(startend_1 == fr.data[0]);
        VA_ASSERT(startend_2 == fr.size + startend_1 - 1);
           
        /* transform to ns (0-4) format */
        while (*p != 0)
        {
            *p = tolower(*p);
            p++;
        }

        rd_res_t = nuc2nuc(read_t);
        VA_ASSERT(rd_res_t.size == read_t.size);
        read_t = rd_res_t;

        /* find if there are any gaps in the read */
        p = read_t.data;

        for (i = startend_1; i < fr.data[fr.size - 1]; i++)
            nr_gaps += (read_t.data[i] == 4) ? 1 : 0;

        s1 = startend_2 + nr_gaps - startend_1 + 1; 

        VA_ASSERT(s1 <= read_t.size);
        read_t.size = s1;
        for (i = 0; i < s1; i++)
            read_t.data[i] = read_t.data[i + startend_1 - 1];

        /* verify that we kept all the nucleotides */
        n = 0;
        for (i = 0; i < read_t.size; i++)
            if (read_t.data[i] != 4)
                n++;
       
        VA_ASSERT(n == fr.size);
        free(fr.data);

        /* finally assign it */
        if (read->strand != '+')
        {
            VA_VecU8 readc_t;

            fliplrU8(read_t); 
            readc_t = get_complement(read_t);
            free(read_t.data);
            read_t = readc_t;
        }
        read->letseq = read_t;
    }
    else
    {
        /* color-space is more complicated */
        VA_VecU8 colorref;
	VA_VecU8 linker;
        VA_VecU8 inlinker;
        VA_VecU8 read2_t;
        VA_UINT8 * p;
        
        inlinker = va_allocVecU8(1);
        inlinker.data[0] = tolower(read_t.data[0]);
        linker = nuc2nuc(inlinker);
        free(inlinker.data);
        inlinker.data = NULL;

        VA_ASSERT(read_t.size > 1); 
        read2_t = va_allocVecU8(read_t.size - 1);
        memcpy(read2_t.data, &read_t.data[1], read2_t.size);
        memcpy(&read_t, &read2_t, sizeof(read_t));
        
        /* translate to */
        for (i = 0; i < read_t.size; i++)
        {
            if (read_t.data[i] == '-')
                read_t.data[i] = '4';
            read_t.data[i] -= '0';
        } 

        colorref.data = read_t.data;
        colorref.size = startend_1;

        /* translate through the beginning of the read */
        trans = col2nuc(linker.data[0], colorref); 
        free(linker.data);
        VA_ASSERT(trans.size > 0);
        trans.data[0] = trans.data[trans.size - 1];
        trans = va_reallocVecU8(trans, 1);      

        /* find if there are any gaps in the read */
        for (i = 0; i < read_t.size; i++)
            nr_gaps += (read_t.data[i] == 4) ? 1 : 0;
  
        if (read->strand == '+')
        {   
            size_t s1 = startend_2 + nr_gaps - (startend_1 + 1);
             
            read->letseq = va_reallocVecU8(trans, 1);
            read->colseq = va_allocVecU8(s1 + 1);
            memcpy(read->colseq.data, &read_t.data[startend_1], 
                read->colseq.size);
        }
        else
        {
            size_t s1 = startend_2 + nr_gaps - startend_1;
            int i;
            int toshift = 1;

            read->letseq = get_complement(trans);
            free(trans.data);
            trans.data = NULL;

            for (i = 0; i < s1; i++)
                read_t.data[i] = read_t.data[i + startend_1];
         
            read_t.size = s1;
            fliplrU8(read_t);

            /* if there are any gaps, they have to be slightly shifted */
            for (i = 0; i < s1; i++)
            {
                if ((read_t.data[i] == 4) && (i > 0))
                {
                    VA_UINT8 t = read_t.data[i - 1];
                    read_t.data[i - 1] = read_t.data[i];
                    read_t.data[i] = t;
                }
            }
        
            read->colseq = va_allocVecU8(s1); 
            memcpy(read->colseq.data, read_t.data, s1 * sizeof(VA_UINT8));
            read->chrstart++;
        }
    }

    /* add insertions */

    /* prepare the start and end of the reads */
    sscanf(curr_read->Gline, scan_Gline, &startend_1, chr, &startend_2);
   //printf("%s got %d and have %d %c\n",curr_read->Gline, startend_1,read->chrstart,read->strand);
    strcpy(genomestring,chr);


    if (read->strand == '-')
    {   
        long t = startend_2;
        startend_2 = startend_1;
        startend_1 = t;
         
        if (!isLetterspace)
            startend_1++;
    }
    VA_ASSERT(startend_1 == read->chrstart);
    VA_ASSERT(startend_2 == read->chrend);
    
    isForward = (read->strand == '+') ? TRUE : FALSE;

    if (isLetterspace)
        seq = read->letseq;
    else
        seq = read->colseq;

    /* cut the chr variable correctly */
    n = 0;
    VA_ASSERT(readstart > 0);
    for (i = readstart; i <= readstart + seq.size - 1; i++)
        chr[n++] = chr[i - 1];
    chr[n] = 0;

    /* 
     * get ins, a vector of insertions, essentially 
     * indexed into the sequence vector.
     */
    if (isForward && isLetterspace)
    {
        VA_VecU8 vchr = va_allocVecU8(strlen(chr));

        memcpy(vchr.data, chr, vchr.size);
        getIns(vchr, read->chrstart, 1, &ins, &read->ins);
    }
    else if (!isForward && isLetterspace)
    {
        VA_VecU8 fchr = va_allocVecU8(strlen(chr));
        
        memcpy(fchr.data, chr, fchr.size);
        fliplrU8(fchr);
        getIns(fchr, read->chrstart, 1, &ins, &read->ins);
    }
    else if (isForward && !isLetterspace)
    {
        VA_VecU8 vchr = va_allocVecU8(strlen(chr));
        
        memcpy(vchr.data, chr, vchr.size);
        getIns(vchr, read->chrstart, 0, &ins, &read->ins);
    }
    else
    {       
        VA_VecU8 fchr = va_allocVecU8(strlen(chr));
        
        memcpy(fchr.data, chr, fchr.size);
        fliplrU8(fchr);
        getIns(fchr, read->chrstart, 1, &ins, &read->ins);
    }

    /* asserting */
    if (isLetterspace || !isForward) {
        VA_ASSERT(startend_2 - startend_1 + 1 == strlen(chr) - ins.size);
    } else {
        VA_ASSERT(startend_2 - startend_1 + 1 == strlen(chr) - ins.size + 1);
    }

    if (read->ins.size > 0)
    {
        ins_assert = va_allocVecI32(read->ins.size);
        for (i = 0; i < ins_assert.size; i++)
            ins_assert.data[i] = read->ins.data[i] + i;
    }
    else
    {
        memset(&ins_assert, 0, sizeof(ins_assert));
    }
    
    if (strlen(chr) > 0)
    {
       chr_assert = va_allocVecU8(strlen(chr));
       memcpy(chr_assert.data, chr, chr_assert.size);
       if (!isForward)
           fliplrU8(chr_assert);
    }
    else
    {
        memset(&chr_assert, 0, sizeof(chr_assert));
    }
        
    if (chr_assert.data != NULL)
       free(chr_assert.data);

    if (ins_assert.data != NULL)
       free(ins_assert.data);

    /* add to the overall insvec */
    if (ins.size > 0)
    {
        VA_VecI32 u = va_unique(ins);
        int j;
 
        for (i = 0; i < u.size; i++)
        {
            int counti = 0;
            int s = 0;

            for (j = 0; j < ins.size; j++)
            {
                if (u.data[i] == ins.data[j])
                    counti++;  
            }

            for (j = 0; j < insvec->size; j++)
            {
                if (u.data[i] == insvec->data[j])
                    s++;
            }

            if (s < counti)
            {
                size_t oldsz = insvec->size;

                if (oldsz > 0)
                    *insvec = va_reallocVecI32(*insvec, oldsz + counti - s);
                else
                    *insvec = va_allocVecI32(counti - s);
            
                for (j = 0; j < counti - s; j++)
                    insvec->data[oldsz + j] = u.data[i];
            }
        } 
        free(ins.data);
    }

    /** cleaning */

    /* another field for old times' sake */
    read->extras.haplome = 1;

    /* adjust ending */
    if (isLetterspace)
    {
        read->chrend = read->chrstart + read->letseq.size - 1;
    }
    else
    {
        if (isForward)
            read->chrend = read->chrstart + read->colseq.size;
        else
            read->chrend = read->chrstart + read->colseq.size - 1;
    }

    if (curr_read->Qline[0] != 0)
    {
        char * p = curr_read->Qline;
        char * q = NULL;
        size_t initsz = readsize;

        read->qvals = va_allocVecU32(initsz);
        read->qvals.size = 0;

        /* parse Qline */
        if ((strlen(p) > 3) && (*p == 'Q') && (*(p+1) == ':') && (*(p+2) == '\t'))
        {
            p += 3;
            while (*p != 0)
            {
                if ((*p == ' ') || (*p == '\t'))
                {
                    p++;
                    continue;
                }

                q = p;
                while (isdigit(*p))
                    p++;
                    
                if ((*p != 0) && (*p != ' ') && (*p != '\n'))
                    break;

                if (*p != 0)
                {
                    *p = 0;
                    p++;
                }
                
                read->qvals.data[read->qvals.size++] = atoi(q);
                VA_ASSERT(read->qvals.size <= initsz);
            }
        }
		//fprintf(stderr,"read_t.data:%s\nread->qvals.size: %i\ninitsz: %i\n",read_t.data,read->qvals.size,initsz);

        VA_ASSERT(read->qvals.size == initsz);
    }
    else
    {
        int j;
        size_t initsz = (read->letseq.size > read->colseq.size) ? 
                        read->letseq.size : read->colseq.size;

        read->qvals = va_allocVecU32(initsz);
        for (j = 0 ; j < read->qvals.size; j++)
            read->qvals.data[j] = 40;
    }

    if (isLetterspace) {
        learn_err_mat(readstring,NULL,genomestring,readstart,readend,readlength);
    } else {
        sscanf(curr_read->Tline, scan_Tline,tline);	
        learn_err_mat(tline,readstring,genomestring,readstart,readend,readlength);
    }
}
    
static void getIns(
   VA_VecU8    chr,
   VA_UINT32   chrstart,
   VA_UINT8    corr,
   VA_VecI32 * ins,
   VA_VecI32 * ins_local)
{
   int n = 0;
   int i;

   VA_ASSERT(ins != NULL);
   VA_ASSERT(ins_local != NULL);

   memset(ins, 0, sizeof(VA_VecI32));
   memset(ins_local, 0, sizeof(VA_VecI32));

   for (i = 0; i < chr.size; i++)
       if (chr.data[i] == '-')
           n++;

   if (n > 0)
   {
       *ins = va_allocVecI32(n);
       *ins_local = va_allocVecI32(n);

       n = 0;
       for (i = 0; i < chr.size; i++)
       {
           if (chr.data[i] == '-')
           {
               ins->data[n] = i + 1;
               ins_local->data[n] = ins->data[n] - (n + 1);
               ins->data[n] = chrstart + ins_local->data[n] - corr;
               n++;
           }
       }
   }
}
