/*
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>
#include "va_assert.h"
#include "va_log.h"
#include "varid.h"


extern double*  cs_error_cnt;
extern double*  cs_error_dpts;
extern unsigned int cs_error_size;
extern double*  ls_error_cnt;
extern double*  ls_error_dpts;
extern unsigned int ls_error_size;



CData_t real_cdata_prep(
    int       doSim, 
    int       isDip)
{
    CData_t cdata;

    memset(&cdata, 0, sizeof(cdata));

    cdata.normodds_cutoff = 0.0;
    cdata.pgenome_cutoff = 0.0;
    cdata.gap_error = 0.03;
    cdata.readsize_cutoff_ls = 999;
    cdata.readsize_cutoff_cs = 999;

    /* needed in emission computations */
    cdata.snp_rate = 0.0001;
    cdata.het_rate = 2.0/3.0;
    cdata.hom_rate = 1.0/3.0;
    cdata.tri_rate = 0.001;

    cdata.itri_rate = 0.0001;
    cdata.ihet_rate = 2.0/3.0;
    cdata.ihom_rate = 1.0/3.0;

    cdata.indel_rate = 0.0001;
    cdata.ins_rate = 1.0/3.0 * cdata.indel_rate;
    cdata.del_rate = 2.0/3.0 * cdata.indel_rate;    
    
    if (doSim)
    {
        cdata.src = SRC_SIM;
        
        /* simulation stuff */
        cdata.coverage = 10;
        cdata.hap_bias = 0.5000;
    }
    else
    {
        cdata.src = SRC_REAL;
    }

    /* diploid ? */
    cdata.mode = (isDip) ? MODE_DIPLOID : MODE_HAPLOID;
    
    /* stuff that should be filled in */
    cdata.ref.size = -1;
    cdata.read_size = -1;

    {
        int i;
 
 
       for (i = 0; i < cs_error_size; i++)
		if (cs_error_dpts[i] > 100)
          		 cdata.error_mat_cs[i] = cs_error_cnt[i]/cs_error_dpts[i];
		else
			cdata.error_mat_cs[i] = 0.01;   
	for (i = 0; i < ls_error_size; i++)
                if (ls_error_dpts[i] > 100)
          		cdata.error_mat_ls[i] = ls_error_cnt[i]/ls_error_dpts[i];
		else
			cdata.error_mat_ls[i] = 0.01;
    }

    cdata.ref.data = NULL;
    return (cdata);
}
